@@ -1,27 +1,33 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+import itertools
 import urlparse
+import re
 import requests
 from bs4 import BeautifulSoup
 
 class WebmentionSend():
 
-    def __init__(self, source, target):
+    LINK_HEADER_RE = re.compile(
+        r'''<([^>]+)>; rel=["'](http://)?webmention(\.org/?)?["']''')
+
+    def __init__(self, source, target, endpoint=None):
         self.source_url = source
         self.target_url = target
-        self.receiver_endpoint = None
+        self.receiver_endpoint = endpoint
 
     def send(self, **kwargs):
         self.error = None
         self.requests_kwargs = kwargs
-        r = self._discoverEndpoint()
-        if not r:
+        if not self.receiver_endpoint:
+            self._discoverEndpoint()
+        if not self.receiver_endpoint:
             return False
         return self._notifyReceiver()
 
     def _discoverEndpoint(self):
-        r = requests.get(self.target_url, **self.requests_kwargs)
+        r = requests.get(self.target_url, verify=False, **self.requests_kwargs)
         if r.status_code != 200:
             self.error = {
                 'code':'BAD_TARGET_URL',
@@ -29,29 +35,40 @@ class WebmentionSend():
                 'request': 'GET %s' % self.target_url,
                 'http_status': r.status_code,
             }
-            return False
+            return
+
+        # look in the headers
+        # XXX: it looks like requests doesn't handle multiple headers with the
+        # same name, e.g. 'Link'. from skimming the code, it looks like the last
+        # one wins. ugh. :/
+        for link in r.headers.get('link', '').split(','):
+            match = self.LINK_HEADER_RE.search(link)
+            if match:
+                self.receiver_endpoint = match.group(1)
+                return
+
+        # look in the content
         html = r.text
         soup = BeautifulSoup(html)
-        tag = soup.find('link', attrs={'rel': 'webmention'})
-        if not tag:
-            # backward compatibility with the webmention 0.1 spec
-            tag = soup.find('link', attrs={'rel': 'http://webmention.org/'})
+        tag = None
+        for name, rel in itertools.product(('link', 'a'), ('webmention', 'http://webmention.org/')):
+            tag = soup.find(name, attrs={'rel': rel})
+            if tag:
+                break
 
-        if tag and tag['href']:
+        if tag and tag.get('href'):
             # add the base scheme and host to relative endpoints
             self.receiver_endpoint = urlparse.urljoin(self.target_url, tag['href'])
-            return True
         else:
             self.error = {
                 'code': 'NO_ENDPOINT',
                 'error_description': 'Unable to discover webmention endpoint.'
             }
-            return False
 
     def _notifyReceiver(self):
         payload = {'source': self.source_url, 'target': self.target_url}
         headers = {'Accept': '*/*'}
-        r = requests.post(self.receiver_endpoint, data=payload, **self.requests_kwargs)
+        r = requests.post(self.receiver_endpoint, verify=False, data=payload, **self.requests_kwargs)
 
         request_str = 'POST %s (with source=%s, target=%s)' % (self.receiver_endpoint, self.source_url, self.target_url)
         if r.status_code / 100 != 2:
